import { merge } from 'rxjs';
import { take, tap, bufferCount } from 'rxjs/operators';

const getLogFunction = (logType) => {
    switch (logType) {
        case 'warn':
            return console.warn;
        case 'error':
            return console.error;
        default:
            return console.info;
    }
};
const getStateDiff = (prevState, state) => {
    const diff = {};
    Object.keys(state).forEach((key) => {
        const prevStateType = typeof prevState[key];
        const stateType = typeof state[key];
        if (prevStateType === 'object' && stateType === 'object') {
            diff[key] = getStateDiff(prevState[key], state[key]);
            return;
        }
        if (prevStateType !== stateType) {
            diff[key] = state[key];
            return;
        }
        if (prevState[key] !== state[key]) {
            diff[key] = state[key];
        }
    });
    if (Object.keys(diff).length === 0) {
        return undefined;
    }
    return diff;
};

/**
 * Decorator used to log state of a @ngrx/component-store
 * Will create a watchState effect
 */
function watchComponentState ({ logType } = {}) {
    return (target) => {
        const name = target.name;
        const logFn = getLogFunction(logType);
        return class extends target {
            constructor() {
                super(...arguments);
                this.watchState = this.effect(() => {
                    let lastTime;
                    const logInitialState$ = this.state$.pipe(take(1), tap((state) => {
                        const time = new Date();
                        const obj = {
                            name,
                            state,
                            time,
                        };
                        logFn(obj);
                        lastTime = time;
                    }));
                    const logDiff$ = this.state$.pipe(bufferCount(2, 1), tap(([prevState, state]) => {
                        const time = new Date();
                        const elaspedTime = lastTime
                            ? time.getTime() - lastTime.getTime()
                            : undefined;
                        const diff = getStateDiff(prevState, state);
                        const obj = {
                            name,
                            prevState,
                            state,
                            diff,
                            time,
                            elaspedTime,
                        };
                        logFn(obj);
                        lastTime = time;
                    }));
                    return merge(logInitialState$, logDiff$);
                });
            }
        };
    };
}

/*
 * Public API Surface of ngrx-watch-component-store
 */

/**
 * Generated bundle index. Do not edit.
 */

export { watchComponentState as WatchComponentState };
//# sourceMappingURL=ngrx-watch-component-store.mjs.map
